#pragma comment(linker, "/version:1.0") // MUST be in the form of major.minor

#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#include "libs\StringIntrinsics.h"
#include "libs\ParseNumber.h"

typedef struct {
	LARGE_INTEGER cbTargetSize;  // desired file size
	UINT cSuccesses;             // successfully resized files
	UINT cErrors;                // errors encountered
} WORKCTX, *PWORKCTX;

VOID __fastcall ResizeFiles( PWORKCTX pctx, PCTSTR pszSearchPath );
VOID __fastcall PrintError( PCTSTR pszPath, DWORD dwErrorCode );

UINT __fastcall EntryMain( UINT argc, PTSTR *argv )
{
	BOOLEAN fShowUsage = FALSE;

	WORKCTX ctx;
	ZeroMemory(&ctx, sizeof(ctx));

	if (argc >= 3 && ParseUnsignedInteger(argv[argc - 1], &ctx.cbTargetSize.QuadPart))
	{
		UINT i;

		for (i = 1; i < argc - 1; ++i)
		{
			ResizeFiles(&ctx, argv[i]);
		}

		// Post-mortem
		_tprintf(TEXT("\t%d file(s) resized"), ctx.cSuccesses);
		if (ctx.cErrors) _tprintf(TEXT(", with %d error(s)"), ctx.cErrors);
		_tprintf(TEXT(".\n"));
	}
	else
	{
		_tprintf(TEXT("Usage: resizefile file [...] size\n"));
		return(~1);
	}

	return(ctx.cErrors);
}

VOID __fastcall ResizeFiles( PWORKCTX pctx, PCTSTR pszSearchPath )
{
	HANDLE hFind;
	WIN32_FIND_DATA finddata;

	// Copy the path to a scratchpad buffer and establish the append position
	TCHAR szPath[MAX_PATH << 2];
	PTSTR pszPathAppend;

	lstrcpy(szPath, pszSearchPath);

	if (pszPathAppend = _tcsrchr(szPath, TEXT('\\')))
		++pszPathAppend;
	else
		pszPathAppend = szPath;

	if ((hFind = FindFirstFile(pszSearchPath, &finddata)) != INVALID_HANDLE_VALUE)
	{
		HANDLE hFile;
		BOOLEAN fSuccess;

		do
		{
			if (!(finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				fSuccess = FALSE;

				lstrcpy(pszPathAppend, finddata.cFileName);

				hFile = CreateFile(
					szPath,
					GENERIC_READ | GENERIC_WRITE,
					FILE_SHARE_READ,
					NULL,
					OPEN_EXISTING,
					0,
					NULL
				);

				if (hFile != INVALID_HANDLE_VALUE)
				{
					if ( SetFilePointerEx(hFile, pctx->cbTargetSize, NULL, FILE_BEGIN) &&
					     SetEndOfFile(hFile) )
					{
						fSuccess = TRUE;
					}

					CloseHandle(hFile);
				}

				if (fSuccess)
				{
					++pctx->cSuccesses;
				}
				else
				{
					++pctx->cErrors;
					PrintError(szPath, GetLastError());
				}
			}

		} while (FindNextFile(hFind, &finddata));

		FindClose(hFind);
	}
	else
	{
		PrintError(pszSearchPath, GetLastError());
	}
}

VOID __fastcall PrintError( PCTSTR pszPath, DWORD dwErrorCode )
{
	_ftprintf(stderr, TEXT("Error: %s"), pszPath);

	if (dwErrorCode)
	{
		PTSTR pszErrorMessage = NULL;

		DWORD cchMessage = FormatMessage(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_FROM_SYSTEM,
			NULL,
			dwErrorCode,
			0,
			(PTSTR)&pszErrorMessage,
			0,
			NULL
		);

		if (cchMessage)
		{
			pszErrorMessage[_tcscspn(pszErrorMessage, TEXT("\r\n"))] = 0;
			_ftprintf(stderr, TEXT(" [%s]"), pszErrorMessage);
		}
		else
		{
			_ftprintf(stderr, TEXT(" [%08X]"), dwErrorCode);
		}

		LocalFree(pszErrorMessage);
	}

	_ftprintf(stderr, TEXT("\n"));
}
